@@ -8,23 +8,25 @@
 """
 
 from MoinMoin import wikiutil
-from shlex import shlex
 from StringIO import StringIO
 from MoinSupport import *
 import re
 
+__version__ = "0.2"
+
 # Regular expressions.
 
 syntax = {
     # At start of line:
-    "rows"      : (r"^==(?!.*?==$)",        re.MULTILINE),              # == not-heading
+    "rows"      : (r"^==(?!.*?==$)[ \t]?",      re.MULTILINE),          # == not-heading ws-excl-nl
+    "continuations" : (r"^\s*\.\.(?!\.)[ \t]?", re.MULTILINE),          # .. ws-excl-nl or .. not-dot
 
     # Within text:
     "markers"   : (r"([{]{3,}|[}]{3,})",    re.MULTILINE | re.DOTALL),  # {{{... or }}}...
     "columns"   : (r"\|\|[ \t]*",           0),                         # || ws-excl-nl
 
     # At start of column text:
-    "column"    : (r"^\s*<(.*?)>\s*(.*)",   re.DOTALL),                 # ws < attributes > ws
+    "column"    : (r"^\s*<([^<].*?)>\s*(.*)",   re.DOTALL),             # ws < not-< attributes > ws
     }
 
 patterns = {}
@@ -123,6 +125,10 @@ def parse(s):
 
                 for text in patterns["columns"].split(row_text):
 
+                    # Replace line continuation strings.
+
+                    text = patterns["continuations"].sub("", text)
+
                     # Only create a new column when a boundary has been found.
 
                     if not column_continued:
@@ -210,6 +216,7 @@ def parse(s):
 
     if columns:
         extractAttributes(columns, row_attrs, table_attrs)
+        span_columns(columns, columnnumber)
 
     return table_attrs, rows
 
@@ -268,122 +275,6 @@ def extractAttributes(columns, row_attrs, table_attrs):
                 table_attrs[name] = value
                 del attrs[name]
 
-def parseAttributes(s, escape=True):
-
-    """
-    Parse the table attributes string 's', returning a mapping of names to
-    values. If 'escape' is set to a true value, the attributes will be suitable
-    for use with the formatter API. If 'escape' is set to a false value, the
-    attributes will have any quoting removed.
-    """
-
-    attrs = {}
-    f = StringIO(s)
-    name = None
-    need_value = False
-
-    for token in shlex(f):
-
-        # Capture the name if needed.
-
-        if name is None:
-            name = escape and wikiutil.escape(token) or strip_token(token)
-
-        # Detect either an equals sign or another name.
-
-        elif not need_value:
-            if token == "=":
-                need_value = True
-            else:
-                attrs[name.lower()] = escape and "true" or True
-                name = wikiutil.escape(token)
-
-        # Otherwise, capture a value.
-
-        else:
-            # Quoting of attributes done similarly to wikiutil.parseAttributes.
-
-            if token:
-                if escape:
-                    if token[0] in ("'", '"'):
-                        token = wikiutil.escape(token)
-                    else:
-                        token = '"%s"' % wikiutil.escape(token, 1)
-                else:
-                    token = strip_token(token)
-    
-            attrs[name.lower()] = token
-            name = None
-            need_value = False
-
-    return attrs
-
-def strip_token(token):
-
-    "Return the given 'token' stripped of quoting."
-
-    if token[0] in ("'", '"') and token[-1] == token[0]:
-        return token[1:-1]
-    else:
-        return token
-
-# Formatting of embedded content.
-# NOTE: Borrowed from EventAggregator.
-
-def getParserClass(request, format):
-
-    """
-    Return a parser class using the 'request' for the given 'format', returning
-    a plain text parser if no parser can be found for the specified 'format'.
-    """
-
-    try:
-        return wikiutil.searchAndImportPlugin(request.cfg, "parser", format or "plain")
-    except wikiutil.PluginMissingError:
-        return wikiutil.searchAndImportPlugin(request.cfg, "parser", "plain")
-
-def getFormatterClass(request, format):
-
-    """
-    Return a formatter class using the 'request' for the given output 'format',
-    returning a plain text formatter if no formatter can be found for the
-    specified 'format'.
-    """
-
-    try:
-        return wikiutil.searchAndImportPlugin(request.cfg, "formatter", format or "plain")
-    except wikiutil.PluginMissingError:
-        return wikiutil.searchAndImportPlugin(request.cfg, "formatter", "plain")
-
-def formatText(text, request, fmt):
-
-    "Format the given 'text' using the specified 'request' and formatter 'fmt'."
-
-    parser_cls = getParserClass(request, request.page.pi["format"])
-    parser = parser_cls(text, request, line_anchors=False)
-    old_fmt = request.formatter
-    request.formatter = fmt
-    try:
-        return redirectedOutput(request, parser, fmt, inhibit_p=True)
-    finally:
-        request.formatter = old_fmt
-
-def redirectedOutput(request, parser, fmt, **kw):
-
-    "A fixed version of the request method of the same name."
-
-    buf = StringIO()
-    request.redirect(buf)
-    try:
-        parser.format(fmt, **kw)
-        if hasattr(fmt, "flush"):
-            buf.write(fmt.flush(True))
-    finally:
-        request.redirect()
-    text = buf.getvalue()
-    buf.close()
-    return text
-
 # Sorting utilities.
 
 def get_sort_columns(s, start=0):
@@ -511,17 +402,20 @@ class Sorter:
 
         return 0
 
-def write_sort_control(request, columnnumber, columns, sort_columns, column_types, table_name, start=0):
+def write_sort_control(request, columnnumber, columns, sort_columns, column_types, table_name, start=0, write=None):
 
     """
     Using the 'request', write a sort control for the given 'columnnumber' in
     the collection of 'columns', using the existing 'sort_columns' and
     'column_types' to construct labels and links that modify the sort criteria,
     and using the given 'table_name' to parameterise the links.
+
+    If the 'write' parameter is specified, use it to write output; otherwise,
+    write output using the request.
     """
 
     fmt = request.formatter
-    write = request.write
+    write = write or request.write
     _ = request.getText
 
     write(fmt.div(1, css_class="sortcolumns"))
@@ -666,11 +560,14 @@ def get_sort_column_output(columns, start=0):
 
 # Common formatting functions.
 
-def formatTable(text, request, fmt, attrs=None):
+def formatTable(text, request, fmt, attrs=None, write=None):
 
     """
     Format the given 'text' using the specified 'request' and formatter 'fmt'.
     The optional 'attrs' can be used to control the presentation of the table.
+
+    If the 'write' parameter is specified, use it to write output; otherwise,
+    write output using the request.
     """
 
     # Parse the table region.
@@ -735,7 +632,7 @@ def formatTable(text, request, fmt, attrs=None):
 
     # Write the table.
 
-    write = request.write
+    write = write or request.write
     write(fmt.table(1, table_attrs))
 
     for rownumber, (row_attrs, columns) in enumerate(table):
@@ -778,7 +675,7 @@ def formatTable(text, request, fmt, attrs=None):
             # Add sorting controls, if appropriate.
 
             if sortable_heading:
-                write_sort_control(request, columnnumber, columns, sort_columns, column_types, table_name)
+                write_sort_control(request, columnnumber, columns, sort_columns, column_types, table_name, write=write)
                 write(fmt.div(0))
 
             write(fmt.table_cell(0))
@@ -787,4 +684,27 @@ def formatTable(text, request, fmt, attrs=None):
 
     write(fmt.table(0))
 
+def formatTableForOutputType(text, request, mimetype, attrs=None, write=None):
+
+    """
+    Format the given 'text' using the specified 'request' for the given output
+    'mimetype'.
+
+    The optional 'attrs' can be used to control the presentation of the table.
+
+    If the 'write' parameter is specified, use it to write output; otherwise,
+    write output using the request.
+    """
+
+    write = write or request.write
+
+    if mimetype == "text/html":
+        write('<html>')
+        write('<body>')
+        fmt = request.html_formatter
+        fmt.setPage(request.page)
+        formatTable(text, request, fmt, attrs, write)
+        write('</body>')
+        write('</html>')
+
 # vim: tabstop=4 expandtab shiftwidth=4
